/*
* Copyright (C) 2017      Ugubok
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AIMPSDK\apiDecoders.h"
#include "AIMPSDK\apiPlugin.h"
#include "AIMPSDK\IUnknownImpl.h"
#include "AIMPSDK\comtypes.h"

#include "NSF_Core.h"
#include "NSF_File.h"

#define  AIMP_FAIL_MESSAGES		// Define this if you want to see message boxes when failure


const PWCHAR PLUGIN_FILEFORMAT_DESCRIPTION = L"NES Sound Format";
const PWCHAR PLUGIN_FILEFORMAT_EXTLIST = L"*.nsf;*.nsfe;";

const PWCHAR PLUGIN_NAME = L"NotSo Fatso";
const PWCHAR PLUGIN_AUTHOR = L"Disch / Ported by Ugubok";
const PWCHAR PLUGIN_SHORT_DESC = L"NSF/NSFe Decoder";
const PWCHAR PLUGIN_FULL_DESC = PLUGIN_FILEFORMAT_EXTLIST;


// Original NSF Core works with files directly, by filename
// But AIMP doesn't provide filenames, it only provides its own file buffer interface
// So we need to subclass the NSF Core and make it works with AIMP stream interface
class CNSFFileModified : public CNSFFile
{
public:
	CNSFFileModified() {
		ASGameTitle = ASArtist = ASCopyright = ASRipper = NULL;
	};

	~CNSFFileModified() {
		Destroy();
	}

	void Destroy();

	int AIMPLoadFile(IAIMPStream* fStream, BYTE needdata, BYTE ignoreversion);

	// Variables below have to contain copy of its szX (ASCII) analog
	PWCHAR	ASGameTitle;
	PWCHAR	ASArtist;
	PWCHAR	ASCopyright;
	PWCHAR	ASRipper;

protected:
	int	AIMPLoadFile_NESM(IAIMPStream* fStream, BYTE needdata, BYTE ignoreversion);
	int	AIMPLoadFile_NSFE(IAIMPStream* fStream, BYTE needdata);
};


class CAIMPNSFDecoderExtension : public IUnknownImpl<IAIMPExtensionAudioDecoder>
{
public:
	CAIMPNSFDecoderExtension(IAIMPCore *Core);
	~CAIMPNSFDecoderExtension();

	// IAIMPExtensionAudioDecoder
	HRESULT WINAPI CreateDecoder(IAIMPStream *Stream, DWORD Flags,
		IAIMPErrorInfo *ErrorInfo, IAIMPAudioDecoder **Decoder);

private:
	IAIMPCore *aimpCore;
};


class CAIMPNSFFileFormat : public IUnknownImpl<IAIMPExtensionFileFormat>
{
public:
	CAIMPNSFFileFormat(IAIMPCore *Core);
	~CAIMPNSFFileFormat();

	// IAIMPExtensionFileFormat
	HRESULT WINAPI GetDescription(IAIMPString **S);
	HRESULT WINAPI GetExtList(IAIMPString **S);
	HRESULT WINAPI GetFlags(DWORD **S);

private:
	IAIMPCore *aimpCore;
};


class CAIMPNSFPlugin : public IUnknownImpl<IAIMPPlugin>
{
public:
	// Information about the plugin
	PWCHAR WINAPI InfoGet(int Index);
	DWORD WINAPI InfoGetCategories();
	// Initialization / Finalization
	HRESULT WINAPI Initialize(IAIMPCore* Core);
	HRESULT WINAPI Finalize();
	// System Notifications
	void WINAPI SystemNotification(int NotifyID, IUnknown* Data);

	bool IsServiceAvailable(IUnknown * provider, REFIID serviceIid);

private:
	IAIMPCore *aimpCore;
	CAIMPNSFDecoderExtension *decoderExtension;
	CAIMPNSFFileFormat *fileformat;
};


class CAIMPNSFDecoder : public IUnknownImpl<IAIMPAudioDecoder>
{
public:
	CAIMPNSFDecoder(IAIMPStream* inStream, IAIMPCore* Core, int* bSuccess);
	~CAIMPNSFDecoder();

	// IAIMPAudioDecoder
	BOOL WINAPI GetFileInfo(IAIMPFileInfo *FileInfo);
	BOOL WINAPI GetStreamInfo(int *SampleRate, int *Channels, int *SampleFormat);

	BOOL WINAPI IsSeekable();
	BOOL WINAPI IsRealTimeStream();

	INT64 WINAPI GetAvailableData();
	INT64 WINAPI GetSize();
	INT64 WINAPI GetPosition();
	BOOL WINAPI SetPosition(const INT64 Value);

	int WINAPI Read(void *Buffer, int Count);

private:
	IAIMPCore* aimpCore;
	CNSFCore nsfCore;
	CNSFFileModified nsfFile;
	BYTE bCoreInitialized;  // 1 When NSF Core is initialized, otherwise 0
	IAIMPStream* fStream;   // AIMP buffer interface that we recieved
};
